local State = GLOBAL.State
local TimeEvent = GLOBAL.TimeEvent
local EventHandler = GLOBAL.EventHandler
local FRAMES = GLOBAL.FRAMES
local ACTIONS = GLOBAL.ACTIONS
local EQUIPSLOTS = GLOBAL.EQUIPSLOTS
local TUNING = GLOBAL.TUNING
local PlayFootstep = GLOBAL.PlayFootstep
local Action = GLOBAL.Action
local ActionHandler = GLOBAL.ActionHandler
local STRINGS = GLOBAL.STRINGS
local _G = GLOBAL

Assets =
{
	Asset("ANIM", "anim/player_idles_wanda.zip"),
	Asset("ANIM", "anim/player_idles_wathgrithr.zip"),
	Asset("ANIM", "anim/player_idles_wendy.zip"),
	Asset("ANIM", "anim/player_idles_willow.zip"),
	Asset("ANIM", "anim/player_idles_warly.zip"),
	Asset("ANIM", "anim/player_idles_wormwood.zip"),
	Asset("ANIM", "anim/hand_wickerbottom.zip"),
	Asset("ANIM", "anim/player_idles_wickerbottom.zip"),
	Asset("ANIM", "anim/player_idles_webber.zip"),
	Asset("ANIM", "anim/player_idles_wes.zip"),
	Asset("ANIM", "anim/player_idles_winona.zip"),
	Asset("ANIM", "anim/player_idles_wurt.zip"),
	Asset("ANIM", "anim/player_idles_wonkey.zip"),
    Asset("ANIM", "anim/player_idles_wolfgang.zip"),
	Asset("ANIM", "anim/player_idles_wx.zip"),
	Asset("ANIM", "anim/player_idles_woodie.zip"),
    Asset("ANIM", "anim/player_idles_wolfgang_skinny.zip"),
    Asset("ANIM", "anim/player_idles_wolfgang_mighty.zip"),
	Asset("ANIM", "anim/player_idles_waxwell.zip"),
	Asset("ANIM", "anim/player_idles_wilson.zip"),
	
	Asset("ANIM", "anim/swap_bernie.zip"),
	
	Asset("ANIM", "anim/player_mime.zip"),
	Asset("ANIM", "anim/player_mime2.zip"),
}

AddComponentPostInit("driver", function(self)
    local _OnUpdate = self.OnUpdate
    function self:OnUpdate(dt)
        _OnUpdate(self, dt)
        if self.vehicle ~= nil and self.vehicle:IsValid() then
            local CameraRight = _G.TheCamera:GetRightVec()
            local CameraDown = _G.TheCamera:GetDownVec()

            local myPos = self.inst:GetPosition()
            local displacement = CameraRight:Cross(CameraDown) * 0.05

            local pos = myPos - displacement

            self.vehicle.Transform:SetPosition(pos:Get())
        end
    end
end)

AddPrefabPostInit("wilson", function(inst)
	local function customidleanimfn_wilson(inst)
		if inst.components.beard and inst.components.beard.bits == 0 then
			return "idle_wilson"
		elseif inst.components.beard and inst.components.beard.bits == 1 then
			return "idle_wilson"
		elseif inst.components.beard and inst.components.beard.bits == 2 then
			return "idle_wilson_beard"
		elseif inst.components.beard and inst.components.beard.bits == 3 then
			return "idle_wilson_beard"
		end
	end
	inst.customidleanim = customidleanimfn_wilson
end)

AddPrefabPostInit("willow", function(inst)
	local function customidlestatefn_willow(inst)
		local item = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
		return item == nil and "willow_funnyidle" or nil
	end
	inst.customidlestate = customidlestatefn_willow
end)

AddPrefabPostInit("wickerbottom", function(inst)
	inst.AnimState:AddOverrideBuild("hand_wickerbottom")
	inst.AnimState:OverrideSymbol("hand_idle_wickerbottom", "hand", "hand_wickerbottom")
	inst.customidleanim = "idle_wickerbottom"
end)

AddPrefabPostInit("woodie", function(inst)
	local function customidleanimfn_woodie(inst)
		local item = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
		return item ~= nil and item.prefab == "lucy" and "idle_woodie" or nil
	end
	inst.customidleanim = customidleanimfn_woodie
end)

AddPrefabPostInit("wathgrithr", function(inst)
	inst.customidleanim = "idle_wathgrithr"
end)

AddPrefabPostInit("warly", function(inst)
	inst.AnimState:AddOverrideBuild("player_idles_warly")
	inst.customidleanim = "idle_warly"
end)

AddPrefabPostInit("wendy", function(inst)
	inst.AnimState:AddOverrideBuild("player_idles_wendy")
	inst.customidleanim = "idle_wendy"
end)

AddPrefabPostInit("wx78", function(inst)
	inst.AnimState:AddOverrideBuild("player_idles_wx")
	inst.customidleanim = "idle_wx"
end)

AddPrefabPostInit("wes", function(inst)
	inst.AnimState:AddOverrideBuild("player_idles_wes")
	inst.customidlestate = "wes_funnyidle"
end)

AddPrefabPostInit("wormwood", function(inst)
	inst.AnimState:AddOverrideBuild("player_idles_wormwood")
	inst.AnimState:OverrideSymbol("hand_idle_wormwood", "wormwood", "hand")
	inst.customidleanim = "idle_wormwood"
end)

AddPrefabPostInit("webber", function(inst)
    inst.customidleanim = "idle_webber"
    inst.AnimState:AddOverrideBuild("player_idles_webber")
end)

AddPrefabPostInit("wilbur", function(inst)
    inst.customidleanim = "idle_wonkey"
end)

AddPrefabPostInit("waxwell", function(inst)
	inst.AnimState:AddOverrideBuild("player_idles_waxwell")
	inst.customidlestate = "waxwell_funnyidle"
end)

AddPrefabPostInit("wolfgang", function(inst)
	local function customidleanimfn_wolfgang(inst)
		if math.random(1,3) == 1 then
			return "idle_wolfgang_mighty"
		elseif math.random(1,3) == 2 then
			return "idle_wolfgang_skinny"
		else
			return "idle_wolfgang"
		end
	end
	inst.customidleanim = customidleanimfn_wolfgang
end)

--- Idle Animations Code ---
local function DoIdleCheck(inst)
	if GLOBAL.IsDLCEnabled(GLOBAL.REIGN_OF_GIANTS) or GLOBAL.IsDLCEnabled(GLOBAL.CAPY_DLC) or GLOBAL.IsDLCEnabled(GLOBAL.PORKLAND_DLC) then
		return (inst.components.temperature:IsFreezing() or
			inst.components.temperature:IsOverheating() or
			inst:HasTag("groggy") or
			inst.components.temperature:GetCurrent() > 70 or
			(inst.components.poisonable and inst.components.poisonable:IsPoisoned()) or
			inst.components.temperature:GetCurrent() < 10 or
			inst.components.temperature:GetCurrent() > 60 or
			inst.components.hunger:GetPercent() < TUNING.HUNGRY_THRESH or
			inst.components.sanity:GetPercent() < .5)
	else
		return (inst.components.temperature:IsFreezing() or
			inst.components.temperature:GetCurrent() < 10 or
			inst.components.hunger:GetPercent() < TUNING.HUNGRY_THRESH or
			inst.components.sanity:GetPercent() < .5)
	end
end

local newIdleAnim = function(inst)
	local anim = inst.customidleanim ~= nil and (type(inst.customidleanim) == "string" and inst.customidleanim or inst:customidleanim()) or nil
	local state = anim == nil and (inst.customidlestate ~= nil and (type(inst.customidlestate) == "string" and inst.customidlestate or inst:customidlestate())) or nil
	if (anim ~= nil or state ~= nil) and inst.components.sanity:IsSane() and not DoIdleCheck(inst) then
		if inst.sg.mem.idlerepeats == nil then
			inst.sg.mem.usecustomidle = math.random() < .5
			inst.sg.mem.idlerepeats = 0
		end
		if inst.sg.mem.idlerepeats > 1 then
			inst.sg.mem.idlerepeats = inst.sg.mem.idlerepeats - 1
		else
			inst.sg.mem.usecustomidle = not inst.sg.mem.usecustomidle
			inst.sg.mem.idlerepeats = inst.sg.mem.usecustomidle and 1 or math.ceil(math.random(2, 5) * .5)
		end
		if inst.sg.mem.usecustomidle then
			if anim ~= nil then
				inst.sg:GoToState("generic_funnyidle")
			else
				inst.sg:GoToState(state)
			end
		else
			inst.sg:GoToState("funnyidle")
		end
	else
		inst.sg:GoToState("funnyidle")
	end
end
local function SGWilsonPostInit(sg)
	sg.states["idle"].ontimeout = newIdleAnim
end
AddStategraphPostInit("wilson", SGWilsonPostInit)
AddStategraphPostInit("wilsonboating", SGWilsonPostInit)

--- Generic Idle Animation ---
AddStategraphState("wilson", GLOBAL.State {
        name = "generic_funnyidle",
        tags = { "idle", "canrotate" },

        onenter = function(inst)
			local anim = inst.customidleanim ~= nil and (type(inst.customidleanim) == "string" and inst.customidleanim or inst:customidleanim()) or nil
			inst.AnimState:PlayAnimation(anim)
        end,

        events =
        {
            EventHandler("animqueueover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },
    }
)
AddStategraphState("wilsonboating", GLOBAL.State {
        name = "generic_funnyidle",
        tags = { "idle", "canrotate", "boating" },

        onenter = function(inst)
			inst.components.driver:SplitFromVehicle()
			local anim = inst.customidleanim ~= nil and (type(inst.customidleanim) == "string" and inst.customidleanim or inst:customidleanim()) or nil
			inst.AnimState:PlayAnimation(anim)
        end,
		
        events =
        {
            EventHandler("animqueueover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
					inst.components.driver:CombineWithVehicle()
                end
            end),
        },
		
        onexit = function(inst)
			inst.components.driver:CombineWithVehicle()
        end,
    }
)

--- Wes Idle Animation ---
AddStategraphState("wilson", GLOBAL.State {
        name = "wes_funnyidle",
        tags = { "idle", "canrotate" },

        onenter = function(inst)
            inst.AnimState:PlayAnimation("idle_wes")
        end,

        timeline =
        {
            TimeEvent(9 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("wes/characters/wes/breath_idle")
            end),
            TimeEvent(26 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("dontstarve/common/balloon_bounce", nil, 4)
            end),
            TimeEvent(42 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("wes/characters/wes/breath_idle")
            end),
            TimeEvent(58 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("dontstarve/common/balloon_bounce", nil, 4)
            end),
            TimeEvent(73 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("dontstarve/common/balloon_pop")
            end),
        },

        events =
        {
            EventHandler("animqueueover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
                end
            end),
        },
    }
)
AddStategraphState("wilsonboating", GLOBAL.State {
        name = "wes_funnyidle",
        tags = { "idle", "canrotate", "boating" },

        onenter = function(inst)
			inst.components.driver:SplitFromVehicle()
            inst.AnimState:PlayAnimation("idle_wes")
        end,

        timeline =
        {
            TimeEvent(9 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("wes/characters/wes/breath_idle")
            end),
            TimeEvent(26 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("dontstarve/common/balloon_bounce", nil, 4)
            end),
            TimeEvent(42 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("wes/characters/wes/breath_idle")
            end),
            TimeEvent(58 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("dontstarve/common/balloon_bounce", nil, 4)
            end),
            TimeEvent(73 * FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("dontstarve/common/balloon_pop")
            end),
        },

        events =
        {
            EventHandler("animqueueover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
					inst.components.driver:CombineWithVehicle()
                end
            end),
        },
        onexit = function(inst)
			inst.components.driver:CombineWithVehicle()
        end,
    }
)

--- Willow Idle Animation ---
AddStategraphState("wilson", GLOBAL.State {
		name = "willow_funnyidle",
		tags = { "idle", "canrotate" },

		onenter = function(inst)
			inst.AnimState:OverrideSymbol("swap_object", "swap_bernie", "swap_bernie")
			inst.AnimState:OverrideSymbol("swap_object_bernie", "swap_bernie", "swap_bernie_idle_willow")

			inst.AnimState:Show("ARM_carry")
			inst.AnimState:Hide("ARM_normal")

			inst.AnimState:PlayAnimation("item_out")
			inst.AnimState:PushAnimation("idle_willow")
			inst.AnimState:PushAnimation("item_in", false)
		end,

		events =
		{
			EventHandler("animover", function(inst)
				if inst.AnimState:AnimDone() then
					inst.AnimState:Hide("ARM_carry")
					inst.AnimState:Show("ARM_normal")
					inst.sg:GoToState("idle")
				end
			end),
            EventHandler("equip", function(inst)
				inst.AnimState:Show("ARM_carry")
				inst.AnimState:Hide("ARM_normal")
				inst.sg:GoToState("idle")
			end),
            EventHandler("unequip", function(inst)
				inst.AnimState:Hide("ARM_carry")
				inst.AnimState:Show("ARM_normal")
				inst.sg:GoToState("idle")
			end),
		},
		
        onexit = function(inst)
			if inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) ~= nil then
				inst.AnimState:Show("ARM_carry")
				inst.AnimState:Hide("ARM_normal")
			end
        end,
	}
)
AddStategraphState("wilsonboating", GLOBAL.State {
		name = "willow_funnyidle",
		tags = { "idle", "canrotate", "boating" },

		onenter = function(inst)
			inst.components.driver:SplitFromVehicle()
			
			inst.AnimState:OverrideSymbol("swap_object", "swap_bernie", "swap_bernie")
			inst.AnimState:OverrideSymbol("swap_object_bernie", "swap_bernie", "swap_bernie_idle_willow")
			
			inst.AnimState:Show("ARM_carry")
			inst.AnimState:Hide("ARM_normal")
			
			inst.AnimState:PlayAnimation("item_out")
			inst.AnimState:PushAnimation("idle_willow")
			inst.AnimState:PushAnimation("item_in", false)
		end,

        events =
        {
			EventHandler("animover", function(inst)
				if inst.AnimState:AnimDone() then
					inst.AnimState:Hide("ARM_carry")
					inst.AnimState:Show("ARM_normal")
					inst.sg:GoToState("idle")
					inst.components.driver:CombineWithVehicle()
				end
			end),
            EventHandler("equip", function(inst)
				inst.AnimState:Show("ARM_carry")
				inst.AnimState:Hide("ARM_normal")
				inst.sg:GoToState("idle")
			end),
            EventHandler("unequip", function(inst)
				inst.AnimState:Hide("ARM_carry")
				inst.AnimState:Show("ARM_normal")
				inst.sg:GoToState("idle")
			end),
        },
        onexit = function(inst)
			if inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) ~= nil then
				inst.AnimState:Show("ARM_carry")
				inst.AnimState:Hide("ARM_normal")
			end

			inst.components.driver:CombineWithVehicle()
        end,
	}
)

--- Maxwell Idle Animation ---
AddStategraphState("wilson", GLOBAL.State {
		name = "waxwell_funnyidle",
		tags = { "idle", "canrotate" },

		onenter = function(inst)
			if math.random(1,3) >= 2 then
				inst.AnimState:PlayAnimation(math.random() < .7 and "idle_waxwell" or "idle2_waxwell")
			else
				inst.AnimState:PlayAnimation("idle3_waxwell")
			end
		end,

		events =
		{
			EventHandler("animover", function(inst)
				if inst.AnimState:AnimDone() then
					inst.sg:GoToState("idle")
				end
			end),
		},
	}
)
AddStategraphState("wilsonboating", GLOBAL.State {
		name = "waxwell_funnyidle",
		tags = { "idle", "canrotate" },

		onenter = function(inst)
			inst.components.driver:SplitFromVehicle()
			inst.AnimState:PlayAnimation(math.random() < .7 and "idle_waxwell" or "idle2_waxwell")
		end,

        events =
        {
            EventHandler("animover", function(inst)
                if inst.AnimState:AnimDone() then
                    inst.sg:GoToState("idle")
					inst.components.driver:CombineWithVehicle()
                end
            end),
        },
        onexit = function(inst)
			inst.components.driver:CombineWithVehicle()
        end,
	}
)

--- Wes Mime Animations ---
AddStategraphState("wilson", GLOBAL.State {
        name = "mime",
        tags = {"idle", "talking"},
        
        onenter = function(inst)
            inst.components.locomotor:Stop()
            
            for k = 1, math.random(2,3) do
				local aname = "mime" .. tostring(math.random(13))
				if k == 1 then
					inst.AnimState:PlayAnimation(aname, false)
				else
					inst.AnimState:PushAnimation(aname, false)
				end
            end
        end,
        
        events=
        {
            EventHandler("animqueueover", function(inst) inst.sg:GoToState("idle") end),
        },
    }
)
AddStategraphState("wilsonboating", GLOBAL.State {
        name = "mime",
        tags = {"idle", "talking"},
        
        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.components.driver:SplitFromVehicle()

            for k = 1, math.random(2,3) do
                local aname = "mime" .. tostring(math.random(13))
                if k == 1 then
                    inst.AnimState:PlayAnimation(aname, false)
                else
                    inst.AnimState:PushAnimation(aname, false)
                end
            end
        end,
        
        events=
        {
            EventHandler("animqueueover", function(inst)
				inst.sg:GoToState("idle")
				inst.components.driver:CombineWithVehicle()
			end),
        },
        onexit = function(inst)
			inst.components.driver:CombineWithVehicle()
        end,
    }
)
